clear all

%% Default setting
system_res = 0.61*1.064/0.75;       %system_res = 0.61*sysConfig.wavelength/sysConfig.NA
cell_resolution = 30; %in um
image_resolution = 141; %in pixel
Pix2Micrometer = cell_resolution/image_resolution;
LocalRange = round(system_res/Pix2Micrometer);
LocalRange = LocalRange+1-rem(LocalRange,2);
fileNum = 1;
sysConfig.wavelength = 1.064;


for fileid = 1:3804   %% Load file
    name = sprintf('%01d_info.mat',fileid);
    load(name);
    [L,n] = bwlabel(cell.mask);
    mask = cell.mask;
    if n > 1
        mask = L == 1;
    end
    
    %% Define variables %%
    feature = [];
    
    %% Features from mask
%     mask = cell.mask;
    maskprops = regionprops(mask,'Area','Centroid','MajorAxisLength','MinorAxisLength');
    area_pix = maskprops.Area;
    [ xLoc, yLoc ] = meshgrid( 1:size(mask,2), 1:size(mask,1));
    CenterOfMask = fliplr(maskprops.Centroid);
    proj_areaTemp = maskprops.Area*Pix2Micrometer^2;
    proj_area = proj_areaTemp;
    CellWidth = maskprops.MinorAxisLength;
    CellLength = maskprops.MajorAxisLength;
    aspectRatio = CellWidth/CellLength;
    
    %% Computation of Volume of cells by slicing method
        C = num2cell(mask,1);
        h = 1;
        VolVector = [];
        for k = 1:numel(C)
            if(sum(C{1,k})~=0)
                vec = C{k};
                radius = sum(C{1,k})/2;
                VolAccTemp = pi*radius^2*h*Pix2Micrometer^3;
                VolVector = [VolVector,VolAccTemp];
            end
        end
        VolAccTemp = sum(VolVector);

    %% Features from BF contrast
    qcellt = cell.BF;
    
    [ AngleMap, distanceMap ] = cart2pol(xLoc-CenterOfMask(2), yLoc-CenterOfMask(1));
    maskedge = bwboundaries(mask);
    maskedge = maskedge{1};
    edgeDis = sqrt(sum(bsxfun(@minus,maskedge,CenterOfMask).^2,2));
    meanEdgeDisTemp = mean(edgeDis); 
    meanEdgeDis(fileNum,1) = meanEdgeDisTemp;
    distanceMap = distanceMap./meanEdgeDisTemp;
    
    amp = sqrt(qcellt);
    cell_body = amp(mask==1);
    
    % Absorption density (opacity)
    absop_density(fileNum,1) = sum(max(cell_body)-cell_body)/proj_areaTemp;
    
    % Statistics from Ampitude map
    Amp_var(fileNum,1) = var(cell_body(:));
    Amp_skewness(fileNum,1) = skewness(cell_body(:),0);
    Amp_kurtosis(fileNum,1) = kurtosis(cell_body(:),0);
    Amp_range(fileNum,1) = range(cell_body);
    
    % Maximum Ampitude
    peak_amp(fileNum,1) = min(cell_body);
    
    % Maximum absorption
    peak_absorp(fileNum,1) = max(max(cell_body)-cell_body);
    
    feature_bf = [meanEdgeDis,absop_density,Amp_var,Amp_skewness,Amp_kurtosis,Amp_range,peak_amp,peak_absorp];
    feature = [feature,feature_bf];
       
    %% Features from BF texture
    % coarse texture (Entrpopy)
    bf_entropy = entropyfilt(qcellt, ones(LocalRange*2-1));
    bfEntr_cell = bf_entropy(mask == 1);
    
    bfEntr_mean(fileNum,1) = mean(bfEntr_cell);
    bfEntr_var(fileNum,1) = var(bfEntr_cell, 0);
    bfEntr_skew(fileNum,1) = skewness(bfEntr_cell,0);
    bfEntr_kur(fileNum,1) = kurtosis(bfEntr_cell,0);
    bfEntr_range(fileNum,1) = range(bfEntr_cell);
    bfEntr_peak(fileNum,1) = max(bfEntr_cell);
    bfEntr_min(fileNum,1) = min(bfEntr_cell);
    
    regProp = regionprops(mask, bf_entropy,'WeightedCentroid');
    bfEntr_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    bfEntr_distance(fileNum,1) = sumabs(bf_entropy.*mask.*distanceMap)./proj_areaTemp;
    
    feature_bftextcoarse = [bfEntr_mean, bfEntr_var, bfEntr_skew, bfEntr_kur, bfEntr_range, bfEntr_peak, bfEntr_min, bfEntr_CenDis, bfEntr_distance ];
    
    % Fine texture
    bf_std = stdfilt(qcellt, ones(LocalRange));
    bfstd_cell = bf_std(mask == 1);

    bfstd_mean(fileNum,1) = mean(bfstd_cell);
    bfstd_var(fileNum,1) = var(bfstd_cell, 0);
    bfstd_skew(fileNum,1) = skewness(bfstd_cell,0);
    bfstd_kur(fileNum,1) = kurtosis(bfstd_cell,0);
    bfstd_range(fileNum,1) = range(bfstd_cell);
    
    bfstd_peak(fileNum,1) = max(bfstd_cell);
    bfstd_min(fileNum,1) = min(bfstd_cell);
    
    regProp = regionprops(mask, bf_std,'WeightedCentroid');
    bfstd_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    bfstd_distance(fileNum,1) = sumabs(bf_std.*mask.*distanceMap)/proj_area;
    
    feature_bftextfine = [bfstd_mean, bfstd_var, bfstd_skew, bfstd_kur, bfstd_range, bfstd_peak, bfstd_min, bfstd_CenDis, bfstd_distance];
    
    % Fibermetric
    bfFib = fibermetric(qcellt,'ObjectPolarity','dark');
    % bfFib_cell = bfFib(mask==1);
    regProp = regionprops(mask, bfFib,'WeightedCentroid');
    bfFib_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;    
    bfFib_distance(fileNum,1) = sumabs(bfFib.*mask.*distanceMap)/proj_area;
    bfFib_75p(fileNum,1) = sum(bfFib(mask == 1) > 0.75)/area_pix;
    bfFib_50p(fileNum,1) = sum(bfFib(mask == 1) > 0.5)/area_pix;
    
    feature_bftextfib = [bfFib_CenDis, bfFib_distance, bfFib_75p, bfFib_50p];
    
    feature = [feature, feature_bftextcoarse, feature_bftextfine, feature_bftextfib];
    
    %% Features from focus factor
    phase = cell.phase;
    
    Grad = imgradient(qcellt);
    GradPh = imgradient(phase);
    FocusFactor1(fileNum,1) = mean(Grad(mask==1));
    [maxLoc(2), maxloc(1)] = find(Grad==max(Grad(mask==1)));
    FocusFactor2(fileNum,1) = mean(GradPh(mask == 1));
    percen = prctile(Grad(:),90);
    FocusFactor1a(fileNum,1) = mean(Grad(Grad>percen));
    FocusFactor2a(fileNum,1) = mean(GradPh(GradPh>percen));    
    percen = prctile(Grad(:),75);
    FocusFactor1b(fileNum,1) = mean(Grad(Grad>percen));
    FocusFactor2b(fileNum,1) = mean(GradPh(GradPh>percen));      
    FocusFactor3(fileNum,1) = var(qcellt(mask==1))/mean(qcellt(mask==1));
    bgStab(fileNum,1) = var(qcellt(mask==0));
    
    feature_focus = [FocusFactor1, FocusFactor1a, FocusFactor1b, FocusFactor2, FocusFactor2a, FocusFactor2b, FocusFactor3, bgStab];
    feature = [feature, feature_focus];
    %% DMD
    VolMap = (CellWidth/2)^2-(aspectRatio*(xLoc-CenterOfMask(2))).^2-((yLoc-CenterOfMask(1))).^2;
    CellVolMap = real(2*sqrt(mask.*VolMap))*Pix2Micrometer;  % Unit in meter
    RIMap = phase./CellVolMap/2/pi*sysConfig.wavelength;
    RIMap((abs(RIMap)==inf))=0;
    DryMassDensityMap = RIMap/0.2; 
    dry_mass_densityTemp = mean(DryMassDensityMap(mask==1));     % Unit in pg/fl = pico-gram per femto-liter
    dry_mass_density(fileNum,1) = dry_mass_densityTemp;
    dry_mass(fileNum,1) = dry_mass_densityTemp*VolAccTemp;
    dry_mass_var(fileNum,1) = var(DryMassDensityMap(mask==1));
    dry_mass_skewness(fileNum,1) = skewness(DryMassDensityMap(mask==1), 0);
    
    feature_DM = [ dry_mass, dry_mass_density, dry_mass_var, dry_mass_skewness ];
    feature = [feature, feature_DM];
    
    %% Features from QPI
    phase_body = phase(mask==1);
    phase_var(fileNum,1) = var(phase_body(:));
    phase_skewness(fileNum,1) = skewness(phase_body(:),0);
    phase_kurtosis(fileNum,1) = kurtosis(phase_body(:),0); 
    phase_range(fileNum,1) = range(phase_body);
    peak_phase(fileNum,1) = max(phase_body);
    min_phase(fileNum,1) = min(phase_body);
    regProp = regionprops(mask, phase,'WeightedCentroid');
    phase_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    
    feature_qp = [ peak_phase, phase_var, phase_skewness, phase_kurtosis, phase_range, min_phase, phase_CenDis];
    
    % Textuere analysis on phase map
    
    % Cell contrast
    contrast_map = stdfilt(phase, ones(LocalRange)); 
    contrast_cell = contrast_map(mask==1);
    
    texture(fileNum,1) = mean(contrast_cell);
    texture_std(fileNum,1) = std(contrast_cell);
    texture_skew(fileNum,1) = skewness(contrast_cell,0);
    texture_kurt(fileNum,1) = kurtosis(contrast_cell,0);
    regProp = regionprops(mask, contrast_map, 'WeightedCentroid');
    texture_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    texture_distance(fileNum,1) = sumabs(contrast_map.*mask.*distanceMap)/proj_area;
    
    feature_DC = [ texture, texture_std, texture_skew, texture_kurt, texture_CenDis, texture_distance];
        
    % Coarse texture
    qpiEntr_map = entropyfilt(phase,ones(LocalRange*2-1));
    qpiEntr_cell = qpiEntr_map(mask==1);
    
    qpiEntr_mean(fileNum,1) = mean(qpiEntr_cell);
    qpiEntr_var(fileNum,1) = var(qpiEntr_cell);
    qpiEntr_skew(fileNum,1) = skewness(qpiEntr_cell);
    qpiEntr_kurt(fileNum,1) = kurtosis(qpiEntr_cell);
    regProp = regionprops(mask, qpiEntr_map, 'WeightedCentroid');
    qpiEntr_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    qpiEntr_distance(fileNum,1) = sumabs(qpiEntr_map.*mask.*distanceMap)/proj_area;
    feature_qpCoarseText = [qpiEntr_mean, qpiEntr_var, qpiEntr_skew, qpiEntr_kurt, qpiEntr_CenDis, qpiEntr_distance]; 
    
    % Fibrous texture
    qpiFib_map = fibermetric(phase);
    regProp = regionprops(mask, qpiFib_map,'WeightedCentroid');
    qpiFib_CenDis(fileNum,1) = sqrt(sum((regProp.WeightedCentroid - maskprops.Centroid).^2))*Pix2Micrometer;
    qpiFib_distance(fileNum,1) = sumabs(qpiFib_map.*mask.*distanceMap)/proj_area;
    qpiFib_75p(fileNum,1) = sum(qpiFib_map(mask == 1) > 0.75)/area_pix;
    qpiFib_50p(fileNum,1) = sum(qpiFib_map(mask == 1) > 0.5)/area_pix;
    feature_qpFibText = [qpiFib_CenDis, qpiFib_distance, qpiFib_75p, qpiFib_50p];
    
    % Spatial distribution of phase
    % Distance distribution
    masked_phase = mask.*phase; 
    distanceMap = distanceMap*Pix2Micrometer; %  [ B, I ] = sort(distanceMap(:)); figure; plot(B, masked_phase(I))
    SPhaseMean_R(fileNum,1) = sumabs(masked_phase.*distanceMap)/sumabs(masked_phase);
    SPhaseVar_RTemp = sumabs(masked_phase.*distanceMap.^2)/(sumabs(masked_phase)-1);
    SPhaseVar_R(fileNum,1) = SPhaseVar_RTemp;
    SPhaseSkew_R(fileNum,1) = sumabs(masked_phase.*distanceMap.^3)/(sumabs(masked_phase)-1)/SPhaseVar_RTemp;
    % Angle distribution
    [ angleList, List ] = sort(AngleMap(:));
    phaseList = masked_phase(List);
    [ angleList, ListOrder ] = unique(angleList);
    phaseList = phaseList(ListOrder);
    AngularFreq = linspace(min(angleList(:)),max(angleList(:)), 3600);
    phaseInAngle = interp1(angleList, phaseList, AngularFreq, 'nearest');
    phaseInAngleFreq = fftshift(abs(fft(phaseInAngle)));
    SPhaseVar_angTemp = sum(phaseInAngleFreq.*AngularFreq.^2)/(sum(phaseInAngleFreq)-1);
    SPhaseVar_ang(fileNum,1) = SPhaseVar_angTemp;
    SPhaseKurt_ang(fileNum,1) = sum(phaseInAngleFreq.*AngularFreq.^4)/(sum(phaseInAngleFreq)-1)/SPhaseVar_angTemp;
    feature_qpDistri = [SPhaseMean_R, SPhaseVar_R, SPhaseSkew_R, SPhaseVar_ang, SPhaseKurt_ang];
    
    feature = [feature, feature_qp, feature_DC, feature_qpCoarseText, feature_qpFibText, feature_qpDistri];
    
    %% Add label %%
    file_id(fileNum,1) = fileid;
    feature_wLabel = [file_id,feature];
    
    fileNum = fileNum + 1;
end
